package org.vaadin.peter.buttongroup.client;

import java.util.LinkedList;
import java.util.List;

import com.google.gwt.user.client.ui.HorizontalPanel;
import com.vaadin.client.ui.VButton;

/**
 * VButtonGroup is client side implementation of the ButtonGroup widget.
 * 
 * @author Peter Lehto / Vaadin Oy
 */
public class VButtonGroup extends HorizontalPanel {

    private static String leftButtonCss = "v-button-left";
    private static String middleButtonCss = "v-button-middle";
    private static String rightButtonCss = "v-button-right";

    private final List<VButton> buttons;

    public VButtonGroup() {
        setStyleName("v-buttongroup");

        buttons = new LinkedList<VButton>();
    }

    public void addButton(VButton button) {
        buttons.add(button);
        add(button);
    }

    public void removeAllButtons() {
        buttons.clear();
        clear();
    }

    public void updateButtonCssClasses() {
        removeOldButtonCss();
        addNewButtonCss();
    }

    private void removeOldButtonCss() {
        for (VButton button : buttons) {
            button.removeStyleName(leftButtonCss);
            button.removeStyleName(middleButtonCss);
            button.removeStyleName(rightButtonCss);
        }
    }

    private void addNewButtonCss() {
        if (getNumberOfVisibleButtons() <= 1) {
            return;
        }

        if (getNumberOfVisibleButtons() == 2) {
            VButton left = getLeftmostVisibleButton();
            VButton right = getRightmostVisibleButton();

            left.addStyleName(leftButtonCss);
            right.addStyleName(rightButtonCss);
            return;
        }

        if (getNumberOfVisibleButtons() > 2) {
            VButton left = getLeftmostVisibleButton();
            VButton right = getRightmostVisibleButton();

            left.addStyleName(leftButtonCss);
            right.addStyleName(rightButtonCss);

            for (VButton button : buttons) {
                if (!button.equals(left) && !button.equals(right)) {
                    button.addStyleName(middleButtonCss);
                }
            }
        }
    }

    private VButton getLeftmostVisibleButton() {
        for (VButton button : buttons) {
            if (button.isVisible()) {
                return button;
            }
        }

        return null;
    }

    private VButton getRightmostVisibleButton() {
        for (int i = buttons.size() - 1; i > 0; i--) {
            if (buttons.get(i).isVisible()) {
                return buttons.get(i);
            }
        }

        return null;
    }

    private int getNumberOfVisibleButtons() {
        int visibleButtons = 0;

        for (VButton button : buttons) {
            if (button.isVisible()) {
                visibleButtons += 1;
            }
        }

        return visibleButtons;
    }
}
